@@ -8,10 +8,9 @@ from tornado import gen
 
 from distributed import Client
 from distributed.client import _as_completed, as_completed, _first_completed
-from distributed.compatibility import Empty
+from distributed.compatibility import Empty, StopAsyncIteration, Queue
 from distributed.utils_test import cluster, gen_cluster, inc
-from distributed.utils_test import loop # flake8: noqa
-from distributed.compatibility import Queue
+from distributed.utils_test import loop  # noqa: F401
 
 
 @gen_cluster(client=True)
@@ -38,8 +37,10 @@ def test_as_completed(loop):
             z = c.submit(inc, 1)
 
             seq = as_completed([x, y, z])
+            assert seq.count() == 3
             assert isinstance(seq, Iterator)
             assert set(seq) == {x, y, z}
+            assert seq.count() == 0
 
             assert list(as_completed([])) == []
 
@@ -127,3 +128,43 @@ def test_as_completed_cancel(loop):
 
             with pytest.raises(Empty):
                 ac.queue.get(timeout=0.1)
+
+            assert list(as_completed([x, y, x])) == [y]
+
+
+def test_as_completed_cancel_last(loop):
+    with cluster() as (s, [a, b]):
+        with Client(s['address'], loop=loop) as c:
+            w = c.submit(sleep, 0.3)
+            x = c.submit(inc, 1)
+            y = c.submit(sleep, 0.3)
+
+            @gen.coroutine
+            def _():
+                yield gen.sleep(0.1)
+                yield w.cancel(asynchronous=True)
+                yield y.cancel(asynchronous=True)
+
+            loop.add_callback(_)
+
+            ac = as_completed([x, y])
+            result = list(ac)
+
+            assert result == [x]
+
+
+@gen_cluster(client=True)
+def test_async_for_py2_equivalent(c, s, a, b):
+    futures = c.map(sleep, [0.01] * 3, pure=False)
+    seq = as_completed(futures)
+    x = yield seq.__anext__()
+    y = yield seq.__anext__()
+    z = yield seq.__anext__()
+
+    assert x.done()
+    assert y.done()
+    assert z.done()
+    assert x.key != y.key
+
+    with pytest.raises(StopAsyncIteration):
+        yield seq.__anext__()
